@@ -4,7 +4,7 @@
 modname = 'seda'
 distname = 'cubicweb-seda'
 
-numversion = (0, 8, 0)
+numversion = (0, 17, 6)
 version = '.'.join(str(num) for num in numversion)
 
 license = 'LGPL'
@@ -14,14 +14,14 @@ description = 'Data Exchange Standard for Archival'
 web = 'http://www.cubicweb.org/project/%s' % distname
 
 __depends__ = {
-    'cubicweb': '>= 3.24',
+    'cubicweb': '>= 3.25.4, < 3.27',
     'six': '>= 1.4.0',
-    'cubicweb-eac': None,
-    'cubicweb-skos': '>= 0.12.1',
-    'cubicweb-compound': '>= 0.4',
+    'cubicweb-eac': '>= 0.8.3, < 0.9.0',
+    'cubicweb-skos': '>= 1.6.0',
+    'cubicweb-compound': '>= 0.7',
     'cubicweb-relationwidget': '>= 0.4',
     'cubicweb-squareui': None,
-    'pyxst': None,
+    'pyxst': '>= 0.3.2',
     'rdflib': '>= 4.1',
 }
 __recommends__ = {}
