@@ -1,4 +1,4 @@
-# copyright 2016-2017 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr -- mailto:contact@logilab.fr
 #
 # This program is free software: you can redistribute it and/or modify it under
@@ -22,19 +22,19 @@ from logilab.common.registry import objectify_predicate
 from logilab.common.decorators import monkeypatch
 
 from cubicweb import tags, _
-from cubicweb.predicates import is_instance
+from cubicweb.predicates import is_instance, match_form_params, specified_etype_implements
 from cubicweb.view import EntityView
 from cubicweb.web.views import autoform, baseviews, tabs, uicfg
 
 from cubicweb_compound import views as compound
-from cubes.relationwidget import views as rwdg
+from cubicweb_relationwidget import views as rwdg
 
 from ..xsd import un_camel_case
 from ..entities import (is_full_seda2_profile, full_seda2_profile,
                         simplified_profile, parent_and_container)
 from ..entities.itree import parent_archive_unit
 from . import (CONTENT_ETYPE, add_subobject_link,
-               rtags_from_xsd_element, rtags_from_rtype_role_targets, copy_rtag, has_rel_perm)
+               rtags_from_xsd_element, rtags_from_rtype_role_targets, has_rel_perm)
 from . import clone, viewlib, widgets
 from . import uicfg as sedauicfg  # noqa - ensure those rules are defined first
 
@@ -96,7 +96,7 @@ def is_simple_keyword(cls, req, rset=None, entity=None, **kwargs):
         unit_type = req.form['keyword_type']
         return int(unit_type == 'simple_keyword')
     except KeyError:
-        if not entity.reverse_seda_keyword_reference_from:
+        if entity is not None and not entity.reverse_seda_keyword_reference_from:
             return 1
         return 0
 
@@ -261,7 +261,7 @@ class ArchiveUnitTabbedPrimaryView(tabs.TabbedPrimaryView):
         'main_tab',
         _('seda_management_tab'),
         _('seda_archive_units_tab'),
-        _('seda_au_data_objects_refs_tab'),
+        _('seda_data_objects_tab'),
         # Content tabs
         _('seda_identification_tab'),
         _('seda_dates_tab'),
@@ -283,8 +283,8 @@ class ArchiveUnitTabbedPrimaryView(tabs.TabbedPrimaryView):
 
 # main tab for archive unit reference ##########################################
 
-au_ref_pvs = copy_rtag(pvs, __name__,
-                       is_instance('SEDAArchiveUnit') & is_archive_unit_ref())
+au_ref_pvs = pvs.derive(__name__,
+                        is_instance('SEDAArchiveUnit') & is_archive_unit_ref())
 au_ref_pvs.tag_subject_of(
     ('SEDAArchiveUnit', 'seda_alt_archive_unit_archive_unit_ref_id', '*'),
     'attributes')
@@ -451,11 +451,12 @@ class ArchiveUnitAgentsTab(ArchiveUnitContentTab):
     content_vid = 'seda_content_agents'
 
 
-class ContentAgentsView(viewlib.SubObjectsTab):
+class ContentAgentsView(ContentSubObjectsView):
     """Display agents related to an archive unit content."""
 
     __regid__ = 'seda_content_agents'
     __select__ = is_instance(CONTENT_ETYPE)
+    tabid = ArchiveUnitAgentsTab.__regid__
 
     rtype_role_targets = [
         ('seda_writer_from', 'object', None),
@@ -481,11 +482,12 @@ class ArchiveUnitCoverageTab(ArchiveUnitContentTab):
     content_vid = 'seda_content_coverage'
 
 
-class ContentCoverageView(viewlib.SubObjectsTab):
+class ContentCoverageView(ContentSubObjectsView):
     """Display coverage information about an archive unit content."""
 
     __regid__ = 'seda_content_coverage'
     __select__ = is_instance(CONTENT_ETYPE)
+    tabid = ArchiveUnitCoverageTab.__regid__
 
     rtype_role_targets = [('seda_spatial', 'object', None),
                           ('seda_temporal', 'object', None),
@@ -591,7 +593,10 @@ class KeywordBusinessValueEntityView(viewlib.ListItemContentView):
 
 afs.tag_subject_of(('SEDAKeywordReference', 'seda_keyword_reference_to_scheme', '*'),
                    'main', 'attributes')
+affk.set_field_kwargs('SEDAKeywordReference', 'seda_keyword_reference_to_scheme',
+                      help=_('select a type to filter out unrelated vocabularies'))
 affk.set_field_kwargs('SEDAKeywordReference', 'seda_keyword_reference_to',
+                      help=_('select first a vocabulary then type here to prompt auto-completion'),
                       widget=widgets.ConceptAutoCompleteWidget(
                           slave_name='seda_keyword_reference_to',
                           master_name='seda_keyword_reference_to_scheme'))
@@ -603,23 +608,47 @@ affk.set_fields_order('SEDAKeywordReference', ['user_cardinality',
 afs.tag_attribute(('SEDAKeyword', 'keyword_content'), 'main', 'hidden')
 afs.tag_attribute(('SEDAKeywordReference', 'user_cardinality'), 'main', 'hidden')
 # custom afs to handle 'simple keyword'
-kw_simple_afs = copy_rtag(afs, __name__,
-                          is_instance('SEDAKeyword') & is_simple_keyword())
+kw_simple_afs = afs.derive(__name__,
+                           is_instance('SEDAKeyword') & is_simple_keyword())
 kw_simple_afs.tag_attribute(('SEDAKeyword', 'keyword_content'), 'main', 'attributes')
 kw_simple_afs.tag_object_of(
     ('*', 'seda_keyword_reference_from', 'SEDAKeyword'),
     'main', 'hidden')
 # but one ordering is enough to rule them all
-affk.set_fields_order('SEDAKeyword', ['keyword_content', ('seda_keyword_reference_from', 'object')])
+affk.set_fields_order('SEDAKeyword', [('seda_keyword_type_from', 'object'),
+                                      'keyword_content',
+                                      ('seda_keyword_reference_from', 'object')])
+
+affk.set_field_kwargs('SEDAKeywordType', 'seda_keyword_type_to',
+                      widget=widgets.KeywordTypeMasterWidget(
+                          slave_base_name='seda_keyword_reference_to_scheme'))
 
 
 class ComplexKeywordAutomaticEntityForm(autoform.AutomaticEntityForm):
     __select__ = (is_instance('SEDAKeyword') & ~is_simple_keyword())
 
     def should_display_inline_creation_form(self, rschema, existing, card):
-        # 1. force creation of one appraisal/access rule
-        if rschema == 'seda_keyword_reference_from':
+        # force display of type and keyword
+        if not existing and rschema in ('seda_keyword_type_from', 'seda_keyword_reference_from'):
             return True
+        return False
+
+    def should_display_add_new_relation_link(self, rschema, existing, card):
+        return False
+
+
+class ComplexKeywordEditionFormView(autoform.InlineEntityEditionFormView):
+    __select__ = (autoform.InlineEntityEditionFormView.__select__
+                  & is_instance('SEDAKeywordType', 'SEDAKeywordReference')
+                  & ~is_simple_keyword())
+    removejs = None
+
+
+class ComplexKeywordCreationFormView(autoform.InlineEntityCreationFormView):
+    __select__ = (autoform.InlineEntityCreationFormView.__select__
+                  & specified_etype_implements('SEDAKeywordType', 'SEDAKeywordReference')
+                  & ~is_simple_keyword())
+    removejs = None
 
 
 # relations tab ################################################################
@@ -676,8 +705,8 @@ class ReferenceListItemContentView(viewlib.ListItemContentView):
         self.w(u'<div class="clearfix"/>')
 
 
-do_ref_afs = copy_rtag(afs, __name__,
-                       is_instance('SEDADataObjectReference') & typed_reference())
+do_ref_afs = afs.derive(__name__,
+                        is_instance('SEDADataObjectReference') & typed_reference())
 do_ref_afs.tag_attribute(('SEDADataObjectReference', 'user_cardinality'), 'main', 'hidden')
 
 for etype in ('SEDAAltIsPartOfArchiveUnitRefId',
@@ -835,7 +864,12 @@ def _display_attributes(self, entity):
 content_ordered_fields = [
     ('seda_description_level', 'subject'),
     ('seda_title', 'object'),
+    ('seda_start_date', 'object'),
+    ('seda_end_date', 'object'),
     ('seda_description', 'object'),
+    ('seda_originating_agency_from', 'object'),
+    ('seda_transferring_agency_archive_unit_identifier', 'object'),
+    ('seda_system_id', 'object'),
     ('seda_version', 'object'),
     ('seda_type_from', 'object'),
     ('seda_document_type', 'object'),
@@ -883,7 +917,7 @@ for rtype, role in content_ordered_fields:
         novalue_label = _('<no value specified>')
     else:
         novalue_label = ' '
-    vid = 'seda.reledit.complexlink' if 'language' in rtype else 'seda.reledit.text'
+    vid = 'seda.reledit.complexlink' if rtype.endswith('_from') else 'seda.reledit.text'
     if role == 'subject':
         rec.tag_subject_of((CONTENT_ETYPE, rtype, '*'),
                            {'rvid': vid, 'novalue_label': novalue_label})
@@ -960,7 +994,9 @@ class SimplifiedArchiveUnitArchiveUnitsTab(ArchiveUnitArchiveUnitsTab):
 
     __select__ = ArchiveUnitArchiveUnitsTab.__select__ & simplified_profile()
 
-    sub_unit_types = [('unit_content', _('seda_archive_unit_object'))]
+    unit_custom_arguments = [
+        ('unit_type', 'unit_content', _('archive unit (content)')),
+    ]
 
 
 # data objects tab #############################################################
@@ -968,7 +1004,7 @@ class SimplifiedArchiveUnitArchiveUnitsTab(ArchiveUnitArchiveUnitsTab):
 class ArchiveUnitDataObjectReferencesTab(ArchiveUnitSubObjectsTab):
     """Tab for data object references of an archive unit"""
 
-    __regid__ = 'seda_au_data_objects_refs_tab'
+    __regid__ = 'seda_data_objects_tab'
 
     rtype_role_targets = [('seda_data_object_reference', 'object', None)]
 
@@ -1026,10 +1062,19 @@ pvs.tag_object_of(('*', 'seda_data_object_reference', CONTENT_ETYPE),
 afs.tag_object_of(('*', 'seda_data_object_reference', CONTENT_ETYPE),
                   'main', 'hidden')
 
+# add rtags for content archive unit in default afs
+afs.tag_object_of(
+    ('*', 'seda_archive_unit_ref_id_from', 'SEDAAltArchiveUnitArchiveUnitRefId'),
+    'inlined', 'hidden')
+afs.tag_subject_of(
+    ('SEDAAltArchiveUnitArchiveUnitRefId',
+     'seda_seq_alt_archive_unit_archive_unit_ref_id_management', '*'),
+    'inlined', 'inlined')
 
-au_ref_afs = copy_rtag(afs, __name__,
-                       is_instance('SEDAAltArchiveUnitArchiveUnitRefId')
-                       & is_archive_unit_ref())
+# and create a custom one for reference archive unit
+au_ref_afs = afs.derive(__name__,
+                        is_instance('SEDAAltArchiveUnitArchiveUnitRefId')
+                        & is_archive_unit_ref())
 au_ref_afs.tag_object_of(
     ('*', 'seda_archive_unit_ref_id_from', 'SEDAAltArchiveUnitArchiveUnitRefId'),
     'inlined', 'inlined')
@@ -1038,17 +1083,6 @@ au_ref_afs.tag_subject_of(
      'seda_seq_alt_archive_unit_archive_unit_ref_id_management', '*'),
     'inlined', 'hidden')
 
-au_content_afs = copy_rtag(afs, __name__,
-                           is_instance('SEDAAltArchiveUnitArchiveUnitRefId')
-                           & ~is_archive_unit_ref())
-au_content_afs.tag_object_of(
-    ('*', 'seda_archive_unit_ref_id_from', 'SEDAAltArchiveUnitArchiveUnitRefId'),
-    'inlined', 'hidden')
-au_content_afs.tag_subject_of(
-    ('SEDAAltArchiveUnitArchiveUnitRefId',
-     'seda_seq_alt_archive_unit_archive_unit_ref_id_management', '*'),
-    'inlined', 'inlined')
-
 
 class AltArchiveUnitRefIdSimplifiedAutomaticEntityForm(widgets.SimplifiedAutomaticEntityForm):
     """Force display of AltArchiveUnitArchiveUnitRefId's sub-ArchiveUnitRefId or
@@ -1072,35 +1106,5 @@ class ArchiveUnitNoTitleEntityInlinedFormRenderer(widgets.NoTitleEntityInlinedFo
                         & is_archive_unit_ref())))
 
 
-class DataObjectSimplifiedAutomaticEntityForm(widgets.SimplifiedAutomaticEntityForm):
-    """On creation of a BinaryDataObject or PhysicalDataObject's in the context of a simplified
-    profile, add a field to handle the creation of the relation to the archive unit specified in
-    `req.form`.
-    """
-
-    # don't add match_form_params('referenced_by') since it's only specified for creation, not
-    # edition
-    __select__ = (widgets.SimplifiedAutomaticEntityForm.__select__
-                  & is_instance('SEDABinaryDataObject', 'SEDAPhysicalDataObject')
-                  & simplified_profile())
-
-    def inlined_form_views(self):
-        views = super(DataObjectSimplifiedAutomaticEntityForm, self).inlined_form_views()
-        ref_forms = [v.form for v in views if v.rtype == 'seda_data_object_reference_id']
-        if ref_forms:  # may be empty if user has no write access
-            ref_form = ref_forms[0]
-            if not ref_form.edited_entity.has_eid() and not ref_form.posting:
-                ref_form.add_hidden(name='seda_data_object_reference', eidparam=True,
-                                    role='subject',
-                                    value=self._cw.form['referenced_by'])
-        return views
-
-
-class DataObjectReferenceNoTitleEntityInlinedFormRenderer(widgets.NoTitleEntityInlinedFormRenderer):
-    """Don't display any title nor remove link for DataObjectReference in the context of a
-    simplified profile.
-    """
-
-    __select__ = (widgets.NoTitleEntityInlinedFormRenderer.__select__
-                  & is_instance('SEDADataObjectReference')
-                  & simplified_profile())
+copy_afs = afs.derive(__name__, afs.__select__ & match_form_params(vid='copy'))
+copy_afs.tag_subject_of(('*', 'seda_alt_archive_unit_archive_unit_ref_id', '*'), 'main', 'hidden')
