@@ -297,12 +297,6 @@ class _CleanerVisitor(object):
         if len(element.children) == 1:
             assert parent_occ.minimum == parent_occ.maximum == 1
             _jump(self._stack[-2], parent_occ, element.children[0])
-        # skip sequence with maxOccurs = 1 and either minOccurs = 1 or all children have
-        # minOccurs = 0
-        elif parent_occ.maximum == 1 and (parent_occ.minimum == 1
-                                          or all(cocc.minimum == 0 for cocc in element.children)):
-            if not isinstance(self._stack[-2], (graph_nodes.XMLElement, graph_nodes.Alternative)):
-                _replace(self._stack[-2], parent_occ, element.children)
 
     leave_sequence = leave_group
 
