@@ -1,4 +1,4 @@
-# copyright 2016-2017 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr -- mailto:contact@logilab.fr
 #
 # This program is free software: you can redistribute it and/or modify it under
@@ -15,6 +15,7 @@
 # with this program. If not, see <http://www.gnu.org/licenses/>.
 """Generate Yams schema from XSD file. XSD parsing is done using pyxst.
 """
+from __future__ import print_function
 
 try:
     from yams import BASE_TYPES
@@ -23,7 +24,7 @@ except ImportError:
 
 from pyxst.xml_struct import graph_nodes
 
-from xsd import XSDMMapping, un_camel_case
+from .xsd import XSDMMapping, un_camel_case
 
 
 EXT_ETYPES = set(['AuthorityRecord', 'ConceptScheme', 'Concept'])
@@ -81,6 +82,32 @@ SKIP_ATTRS = set([
     'width',
 ])
 
+# list of entity types that may be used multiple times at a same level, and
+# through which relation
+MULTIPLE_CHILDREN = [
+    ('SEDAArchiveUnit', 'seda_archive_unit'),
+    ('SEDABinaryDataObject', 'seda_binary_data_object'),
+    ('SEDAPhysicalDataObject', 'seda_physical_data_object'),
+    ('SEDADataObjectReference', 'seda_data_object_reference'),
+    ('SEDARelatedTransferReference', 'seda_related_transfer_reference'),
+    ('SEDAWriter', 'seda_writer_from'),
+    ('SEDAAddressee', 'seda_addressee_from'),
+    ('SEDARecipient', 'seda_recipient_from'),
+    ('SEDASpatial', 'seda_spatial'),
+    ('SEDATemporal', 'seda_temporal '),
+    ('SEDAJuridictional', 'seda_juridictional'),
+    ('SEDAKeyword', 'seda_keyword'),
+    ('SEDATag', 'seda_tag'),
+    ('SEDAIsVersionOf', 'seda_is_version_of'),
+    ('SEDAReplaces', 'seda_replaces'),
+    ('SEDARequires', 'seda_requires'),
+    ('SEDAIsPartOf', 'seda_is_part_of'),
+    ('SEDAReferences', 'seda_references'),
+    ('SEDAEvent', 'seda_event'),
+    ('SEDACustodialHistoryItem', 'seda_custodial_history_item'),
+    ('SEDARelationship', 'seda_relationship'),
+]
+
 RDEF_CONSTRAINTS = {
     # x-ref constraints
     'seda_data_object_reference_id': 'S container C, O container C',
@@ -131,7 +158,7 @@ RDEF_CONSTRAINTS = {
         '       CACLV seda_mime_type_code_list_version_to CS,'
         '       S container AT)'
         ' OR EXISTS(S container AU, AU is SEDAArchiveUnit, CS scheme_relation_type RT, '
-        '           RT name "seda_mime_type_to")'),
+        '           RT name "file_category")'),
     'seda_encoding_to': (
         'O in_scheme CS, '
         'EXISTS(CACLV seda_encoding_code_list_version_from AT, '
@@ -140,9 +167,12 @@ RDEF_CONSTRAINTS = {
         ' OR EXISTS(S container AU, AU is SEDAArchiveUnit, CS scheme_relation_type RT, '
         '           RT name "seda_encoding_to")'),
     'seda_format_id_to': (
-        'O in_scheme CS, CACL seda_file_format_code_list_version_from AT, '
-        'CACL seda_file_format_code_list_version_to CS, '
-        'S container AT'),
+        'O in_scheme CS, '
+        'EXISTS(CACL seda_file_format_code_list_version_from AT, '
+        '       CACL seda_file_format_code_list_version_to CS, '
+        '       S container AT)'
+        ' OR EXISTS(S container AU, AU is SEDAArchiveUnit, CS scheme_relation_type RT, '
+        '           RT name "file_category")'),
     'seda_data_object_version_to': (
         'O in_scheme CS, CACLV seda_data_object_version_code_list_version_from AT, '
         'CACLV seda_data_object_version_code_list_version_to CS,'
@@ -195,25 +225,20 @@ RDEF_CONSTRAINTS = {
         'O in_scheme CS, S seda_keyword_reference_to_scheme CS'),
 }
 RTYPE_CARDS = {
-    'seda_title': '11',
-    'seda_description': '1?',
-    'seda_comment': '1?',
-    'seda_custodial_history_item': '1*',
     'seda_archive_unit': '?*',
     'seda_binary_data_object': '?*',
+    'seda_comment': '1?',
+    'seda_custodial_history_item': '1*',
+    'seda_description': '1?',
+    'seda_description_level': '1*',
+    'seda_format_id_to': '**',
+    'seda_mime_type_to': '**',
     'seda_physical_data_object': '?*',
+    'seda_title': '11',
 }
 RTYPE_CARD = {
     'seda_custodial_history_item': '*',
 }
-DEFAULT_USER_CARDINALITY = {
-    'SEDAStartDate': '1',
-    'SEDAKeywordReference': '1',
-}
-for rule_type in RULE_TYPES:
-    etype = 'SEDASeq{0}RuleRule'.format(rule_type.capitalize())
-    DEFAULT_USER_CARDINALITY[etype] = '1'
-
 _CARD_TO_CARDS = {
     '1': ['1'],
     '?': ['0..1', '1'],
@@ -309,7 +334,10 @@ class ETypeMapping(object):
 
 class RdefMapping(object):
     """Map a relation to a simple relation definition."""
-    def __init__(self, subjtype, rtype, objtypes, card, composite=None, inlined=False, alias=None):
+
+    def __init__(self, subjtype, rtype, objtypes, card,
+                 composite=None, inlined=False, alias=None,
+                 desc=(), element_name=None):
         assert len(card) == 2, card
         self.subjtype = subjtype
         self.rtype = rtype
@@ -318,6 +346,8 @@ class RdefMapping(object):
         self.composite = composite
         self.inlined = inlined
         self.alias = alias
+        self.desc = desc  # list of text collected under xsd:documentation
+        self.element_name = element_name  # XML tag name
 
     @property
     def final(self):
@@ -368,6 +398,8 @@ class XSYMapping(object):
                     # (it should be systematically added in XSD for the later)
                     if rtype == 'seda_uri' and emapping.etype == 'SEDAAttachment':
                         continue
+                    rdef_options['desc'] = getattr(occ.target, 'desc', ())
+                    rdef_options['element_name'] = getattr(occ.target, 'local_name', None)
                     if role == 'subject':
                         subjtype = current_emapping.etype
                         objtype = target_etype
@@ -380,6 +412,8 @@ class XSYMapping(object):
                     composite = rdef_options.get('composite')
                     if rtype in RTYPE_CARDS:
                         rdef_options['card'] = RTYPE_CARDS[rtype]
+                        if RTYPE_CARDS[rtype][0] not in '?1':
+                            rdef_options['inlined'] = False
                     elif composite == 'subject':
                         rdef_options['card'] = fixedcard + '1'
                     elif composite == 'object':
@@ -421,18 +455,21 @@ class XSYMapping(object):
                     cards = set(cards)
                     emapping_cards = set(emapping.cards)
                     if cards - emapping_cards:
-                        print "# XXX extending cards for", emapping, cards - emapping_cards
+                        print("# XXX extending cards for", emapping, cards - emapping_cards)
                         emapping.cards = sorted(emapping_cards | cards)
         else:
             self.etypes[emapping.etype] = emapping
             self.ordered.append(emapping)
         return emapping
 
-    def map_rdef(self, subjtype, rtype, objtypes, card, composite=None, inlined=False, alias=None):
+    def map_rdef(self, subjtype, rtype, objtypes, card,
+                 composite=None, inlined=False, alias=None,
+                 desc=(), element_name=None):
         """Register a relation definition."""
         objtypes = _ensure_set(objtypes)
         mapping = RdefMapping(subjtype, rtype, objtypes,
-                              card=card, composite=composite, inlined=inlined, alias=alias)
+                              card=card, composite=composite, inlined=inlined, alias=alias,
+                              desc=desc, element_name=element_name)
         self.ordered.append(mapping)
         return mapping
 
@@ -451,8 +488,8 @@ def _merge_mapping(ref_mapping, subjtype, rtype, objtype, composite, card=None):
     assert composite == ref_mapping.composite, (ref_mapping, composite)
     if card is not None:
         if not card == ref_mapping.card[1]:
-            print '# XXX unsupported merge because of incompatible cardinality', subjtype, rtype, \
-                objtype, card, ref_mapping, ref_mapping.card
+            print('# XXX unsupported merge because of incompatible cardinality', subjtype, rtype,
+                  objtype, card, ref_mapping, ref_mapping.card)
         ref_mapping.objtypes.update(_ensure_set(objtype))
     else:
         assert ref_mapping.objtypes == _ensure_set(objtype)
@@ -478,7 +515,7 @@ class CodeGenerator(object):
         """Generator entry point: write generated code for :class:`XSYMapping` into the given stream
         """
         if with_header:
-            stream.write(_PY_HEADER)
+            stream.write(_PY_HEADER.encode('utf-8'))
         self._generate(mapping, stream)
 
     def _generate(self, mapping, stream):
@@ -501,7 +538,7 @@ class YamsSchemaGenerator(CodeGenerator):
     """Yams schema generator"""
 
     def _generate(self, mapping, stream):
-        stream.write('''from yams.buildobjs import EntityType, RelationDefinition
+        stream.write(b'''from yams.buildobjs import EntityType, RelationDefinition
 from yams.buildobjs import String, Boolean
 from cubicweb.schema import RQLConstraint
 from cubicweb_seda.schema import seda_profile_element
@@ -517,7 +554,7 @@ from cubicweb_seda.schema import seda_profile_element
         if mapping.cards:
             cards = mapping.cards
             annotable = len(cards) > 1
-            default_card = DEFAULT_USER_CARDINALITY.get(mapping.etype, cards[0])
+            default_card = '1'
             code = '''\
 @seda_profile_element(cardinalities={cards}, default_cardinality='{default_card}',
                       annotable={annotable})
@@ -577,7 +614,7 @@ def _alias(etype, rtype):
     return (un_camel_case(etype) + '_' + rtype).replace('seda_', '')
 
 
-_PY_HEADER = '''# copyright 2016-2017 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+_PY_HEADER = '''# copyright 2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr -- mailto:contact@logilab.fr
 #
 # This program is free software: you can redistribute it and/or modify it under
