@@ -23,9 +23,9 @@ from cubicweb_seda import dataimport
 class ConcepSchemeImportTC(CubicWebTC):
 
     def test_import_seda_schemes(self):
-        with self.admin_access.client_cnx() as cnx:
+        with self.admin_access.cnx() as cnx:
             dataimport.import_seda_schemes(cnx, lcsv_import=dataimport.lcsv_check)
-            self.assertEqual(len(cnx.find('ConceptScheme')), 18)
+            self.assertEqual(len(cnx.find('ConceptScheme')), 20)
 
 
 if __name__ == '__main__':
