@@ -1,4 +1,4 @@
-# copyright 2016-2017 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+# copyright 2016 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 # contact http://www.logilab.fr -- mailto:contact@logilab.fr
 #
 # This program is free software: you can redistribute it and/or modify it under
@@ -15,19 +15,21 @@
 # with this program. If not, see <http://www.gnu.org/licenses/>.
 """cubicweb-seda unit tests for schema"""
 
-from contextlib import contextmanager
-
-from cubicweb import ValidationError, Unauthorized
 from cubicweb.devtools.testlib import CubicWebTC
 from cubicweb.schema import ERQLExpression, RRQLExpression
 
-import testutils
+from cubicweb_seda import iter_all_rdefs
+
+from cubicweb_seda import testutils
 
 
 class SchemaConceptConstraintsTC(CubicWebTC):
+
+    assertValidationError = testutils.assertValidationError
+
     def setup_database(self):
-        with self.admin_access.client_cnx() as cnx:
-            mt_scheme = testutils.scheme_for_type(cnx, 'seda_mime_type_to', None)
+        with self.admin_access.cnx() as cnx:
+            mt_scheme = testutils.scheme_for_type(cnx, 'file_category', None)
             mt_concept = mt_scheme.add_concept(label=u'text/plain')
             enc_scheme = testutils.scheme_for_type(cnx, 'seda_encoding_to', None)
             enc_concept = enc_scheme.add_concept(label=u'utf-8')
@@ -38,7 +40,7 @@ class SchemaConceptConstraintsTC(CubicWebTC):
             self.enc_concept = enc_concept.eid
 
     def test_code_lists_constraints_simple(self):
-        with self.admin_access.client_cnx() as cnx:
+        with self.admin_access.cnx() as cnx:
             bdo = testutils.create_transfer_to_bdo(cnx)
             transfer = bdo.container[0]
             cnx.create_entity('SEDAMessageDigestAlgorithmCodeListVersion',
@@ -46,9 +48,8 @@ class SchemaConceptConstraintsTC(CubicWebTC):
                               seda_message_digest_algorithm_code_list_version_to=self.enc_scheme)
             cnx.commit()
 
-            bdo.cw_set(seda_algorithm=self.mt_concept)
-            with self.assertRaises(ValidationError) as cm:
-                cnx.commit()
+            with self.assertValidationError(cnx) as cm:
+                bdo.cw_set(seda_algorithm=self.mt_concept)
             self.assertIn('seda_algorithm-subject', cm.exception.errors)
 
             bdo.cw_set(seda_algorithm=self.enc_concept)
@@ -57,26 +58,21 @@ class SchemaConceptConstraintsTC(CubicWebTC):
     def assertMimeTypeConcept(self, bdo):
         cnx = bdo._cw
 
-        cnx.create_entity('SEDAMimeType',
-                          seda_mime_type_from=bdo,
-                          seda_mime_type_to=self.enc_concept)
-        with self.assertRaises(ValidationError) as cm:
-            cnx.commit()
+        with self.assertValidationError(cnx) as cm:
+            bdo.mime_type.cw_set(seda_mime_type_to=self.enc_concept)
         self.assertIn('seda_mime_type_to-subject', cm.exception.errors)
 
-        cnx.create_entity('SEDAMimeType',
-                          seda_mime_type_from=bdo,
-                          seda_mime_type_to=self.mt_concept)
+        bdo.mime_type.cw_set(seda_mime_type_to=self.mt_concept)
         cnx.commit()
 
     def test_archive_transfer_mime_type_constraint(self):
-        with self.admin_access.client_cnx() as cnx:
+        with self.admin_access.cnx() as cnx:
             bdo = testutils.create_transfer_to_bdo(cnx)
             cnx.commit()  # will setup code list version
             self.assertMimeTypeConcept(bdo)
 
     def test_component_archive_unit_mime_type_constraint(self):
-        with self.admin_access.client_cnx() as cnx:
+        with self.admin_access.cnx() as cnx:
             unit, unit_alt, unit_alt_seq = testutils.create_archive_unit(None, cnx=cnx)
             bdo = testutils.create_data_object(unit_alt_seq)
             cnx.commit()
@@ -85,11 +81,10 @@ class SchemaConceptConstraintsTC(CubicWebTC):
     def assertEncodingConcept(self, bdo):
         cnx = bdo._cw
 
-        cnx.create_entity('SEDAEncoding',
-                          seda_encoding_from=bdo,
-                          seda_encoding_to=self.mt_concept)
-        with self.assertRaises(ValidationError) as cm:
-            cnx.commit()
+        with self.assertValidationError(cnx) as cm:
+            cnx.create_entity('SEDAEncoding',
+                              seda_encoding_from=bdo,
+                              seda_encoding_to=self.mt_concept)
         self.assertIn('seda_encoding_to-subject', cm.exception.errors)
 
         cnx.create_entity('SEDAEncoding',
@@ -98,13 +93,13 @@ class SchemaConceptConstraintsTC(CubicWebTC):
         cnx.commit()
 
     def test_archive_transfer_encoding_constraint(self):
-        with self.admin_access.client_cnx() as cnx:
+        with self.admin_access.cnx() as cnx:
             bdo = testutils.create_transfer_to_bdo(cnx)
             cnx.commit()   # will setup code list version
             self.assertEncodingConcept(bdo)
 
     def test_component_archive_unit_encoding_constraint(self):
-        with self.admin_access.client_cnx() as cnx:
+        with self.admin_access.cnx() as cnx:
             unit, unit_alt, unit_alt_seq = testutils.create_archive_unit(None, cnx=cnx)
             bdo = testutils.create_data_object(unit_alt_seq)
             cnx.commit()
@@ -113,16 +108,15 @@ class SchemaConceptConstraintsTC(CubicWebTC):
     def assertDigestAlgorithmConcept(self, bdo):
         cnx = bdo._cw
 
-        with self.assertRaises(ValidationError) as cm:
+        with self.assertValidationError(cnx) as cm:
             bdo.cw_set(seda_algorithm=self.mt_concept)
-            cnx.commit()
         self.assertIn('seda_algorithm-subject', cm.exception.errors)
 
         bdo.cw_set(seda_algorithm=self.enc_concept)
         cnx.commit()
 
     def test_archive_transfer_digest_algorithm_constraint(self):
-        with self.admin_access.client_cnx() as cnx:
+        with self.admin_access.cnx() as cnx:
             bdo = testutils.create_transfer_to_bdo(cnx)
             cnx.commit()  # commit first to get the container
             transfer = bdo.container[0]
@@ -133,7 +127,7 @@ class SchemaConceptConstraintsTC(CubicWebTC):
             self.assertDigestAlgorithmConcept(bdo)
 
     def test_component_archive_unit_digest_algorithm_constraint(self):
-        with self.admin_access.client_cnx() as cnx:
+        with self.admin_access.cnx() as cnx:
             unit, unit_alt, unit_alt_seq = testutils.create_archive_unit(None, cnx=cnx)
             bdo = testutils.create_data_object(unit_alt_seq)
             bdo_type = cnx.find('CWEType', name=u'SEDABinaryDataObject').one()
@@ -146,8 +140,10 @@ class SchemaConceptConstraintsTC(CubicWebTC):
 
 class SchemaTC(CubicWebTC):
 
+    assertValidationError = testutils.assertValidationError
+
     def test_component_archive_unit_rule_constraint(self):
-        with self.admin_access.client_cnx() as cnx:
+        with self.admin_access.cnx() as cnx:
             for rule_type in ('access', 'appraisal'):
                 etype = 'SEDASeq{0}RuleRule'.format(rule_type.capitalize())
                 scheme = testutils.scheme_for_type(cnx, 'seda_rule', etype)
@@ -206,7 +202,7 @@ class SchemaTC(CubicWebTC):
                              'G name IN("managers", "users")')
 
     def test_rule_default_cardinality(self):
-        with self.admin_access.client_cnx() as cnx:
+        with self.admin_access.cnx() as cnx:
             transfer = cnx.create_entity('SEDAArchiveTransfer', title=u'test profile')
             for rule_type in ('access', 'appraisal'):
                 rule_etype = 'SEDA{0}Rule'.format(rule_type.capitalize())
@@ -222,7 +218,7 @@ class SchemaTC(CubicWebTC):
     def test_fti(self):
         # "Reverse" text to be searched in order not to be troubled by other
         # entities that may live in the DB (e.g. Concepts) with similar text.
-        with self.admin_access.client_cnx() as cnx:
+        with self.admin_access.cnx() as cnx:
             transfer = cnx.create_entity('SEDAArchiveTransfer', title=u'Profile')
             unit, unit_alt, unit_alt_seq = testutils.create_archive_unit(
                 transfer, title=u'transfer name'[::-1])
@@ -237,22 +233,80 @@ class SchemaTC(CubicWebTC):
                                        {'search': search[::-1]})
                     self.assertEqual([r for r, in rset.rows], [transfer.eid])
 
+    def test_scheme_code_keyword_type_constraint(self):
+        with self.admin_access.cnx() as cnx:
+            ckt_scheme = testutils.scheme_for_type(cnx, 'seda_keyword_type_to', None)
+            ckt_concept = ckt_scheme.add_concept(label=u'geoname')
+            scheme = testutils.scheme_for_type(cnx, 'seda_mime_type_to', None)
+            concept = scheme.add_concept(label=u'text/plain')
+            cnx.commit()
+
+            with self.assertValidationError(cnx):
+                scheme.cw_set(code_keyword_type=concept)
+
+            scheme.cw_set(code_keyword_type=ckt_concept)
+            cnx.commit()
+
+    def test_container_relation(self):
+        container_rtype = self.schema['container']
+        for rdef in container_rtype.rdefs.values():
+            if rdef.subject == 'SEDAArchiveUnit':
+                self.assertEqual(rdef.cardinality, '?*')
+            else:
+                self.assertEqual(rdef.cardinality, '1*')
+            self.assertEqual(rdef.permissions, {'add': (),
+                                                'delete': (),
+                                                'read': ('managers', 'users', 'guests')})
+
+    def test_reference_to_data_object_is_not_composite(self):
+        self._test_reference_to_data_object_composite(False)
+
+    def test_simplified_reference_to_data_object_is_composite(self):
+        self._test_reference_to_data_object_composite(True)
+
+    def _test_reference_to_data_object_composite(self, simplified):
+        with self.admin_access.cnx() as cnx:
+            transfer = cnx.create_entity('SEDAArchiveTransfer', title=u'test profile',
+                                         simplified_profile=simplified)
+            unit, unit_alt, unit_alt_seq = testutils.create_archive_unit(transfer)
+            bdo = testutils.create_data_object(unit_alt_seq,
+                                               seda_binary_data_object=transfer)
+            cnx.commit()
+            reference = bdo.reverse_seda_data_object_reference_id[0]
+            reference.cw_delete()
+            cnx.commit()
+            self.assertEqual(
+                len(cnx.execute('Any X WHERE X eid %(x)s', {'x': bdo.eid})),
+                0 if simplified else 1)
+
+    def test_delete_archiveunit_with_data_object(self):
+        """Make sure deletion of an archive unit with a binary data object,
+        but not related to an archive transfer is possible.
+        """
+        with self.admin_access.cnx() as cnx:
+            unit, unit_alt, unit_alt_seq = testutils.create_archive_unit(None, cnx=cnx)
+            bdo = testutils.create_data_object(unit_alt_seq)
+            cnx.commit()
+            bdo.cw_clear_all_caches()
+            unit.cw_delete()
+            cnx.commit()
+            self.assertFalse(cnx.execute('Any X WHERE X eid %(x)s', {'x': bdo.eid}))
+
 
 class SecurityTC(CubicWebTC):
 
-    @contextmanager
-    def assertUnauthorized(self, cnx):
-        with self.assertRaises(Unauthorized) as cm:
-            yield cm
+    assertUnauthorized = testutils.assertUnauthorized
+
+    def setup_database(self):
+        with self.admin_access.cnx() as cnx:
+            self.create_user(cnx, login='alice')
             cnx.commit()
-        cnx.rollback()
 
     def test_profile(self):
-        with self.admin_access.repo_cnx() as cnx:
-            self.create_user(cnx, login='alice')
+        with self.admin_access.cnx() as cnx:
             self.create_user(cnx, login='bob')
             cnx.commit()
-        with self.new_access('alice').repo_cnx() as cnx:
+        with self.new_access('alice').cnx() as cnx:
             transfer = cnx.create_entity('SEDAArchiveTransfer', title=u'Alice Profile')
             testutils.create_archive_unit(transfer)
             testutils.create_authority_record(cnx, u'Archival inc.',
@@ -264,7 +318,7 @@ class SecurityTC(CubicWebTC):
                               seda_mime_type_code_list_version_from=transfer,
                               seda_mime_type_code_list_version_to=scheme)
             cnx.commit()
-        with self.new_access('bob').repo_cnx() as cnx:
+        with self.new_access('bob').cnx() as cnx:
             transfer = cnx.entity_from_eid(transfer.eid)
             # modification of the container
             with self.assertUnauthorized(cnx):
@@ -274,7 +328,7 @@ class SecurityTC(CubicWebTC):
             with self.assertUnauthorized(cnx):
                 comment.cw_set(comment=u'You got hacked')
             with self.assertUnauthorized(cnx):
-                cnx.create_entity('SEDAComment', seda_comment=transfer)
+                cnx.create_entity('SEDAArchivalAgreement', seda_archival_agreement=transfer)
             # modification of a relation from the container to a non contained entity
             with self.assertUnauthorized(cnx):
                 testutils.create_authority_record(cnx, name=u'Bob Archival inc.',
@@ -298,8 +352,34 @@ class SecurityTC(CubicWebTC):
             with self.assertUnauthorized(cnx):
                 transfer.cw_delete()
 
+        with self.admin_access.cnx() as cnx:
+            transfer = cnx.entity_from_eid(transfer.eid)
+            # ensure every subobjects permissions depends on top-level
+            # permissions (don't even include managers group)
+            with self.temporary_permissions((self.schema['SEDAArchiveTransfer'],
+                                             {'update': (),
+                                              'delete': ()})):
+                # modification of a contained entity
+                comment = transfer.reverse_seda_comment[0]
+                with self.assertUnauthorized(cnx):
+                    comment.cw_set(comment=u'You got hacked')
+                with self.assertUnauthorized(cnx):
+                    comment.cw_delete()
+                with self.assertUnauthorized(cnx):
+                    cnx.create_entity('SEDAArchivalAgreement', seda_archival_agreement=transfer)
+                # modification of a relation from the container to a non contained entity
+                with self.assertUnauthorized(cnx):
+                    testutils.create_authority_record(cnx, name=u'Bob Archival inc.',
+                                                      reverse_seda_archival_agency=transfer)
+                # deletion of an archive unit
+                with self.assertUnauthorized(cnx):
+                    transfer.archive_units[0].cw_delete()
+                # deletion of the container
+                with self.assertUnauthorized(cnx):
+                    transfer.cw_delete()
+
     def test_archive_unit(self):
-        with self.admin_access.repo_cnx() as cnx:
+        with self.admin_access.cnx() as cnx:
             unit, unit_alt, unit_alt_seq = testutils.create_archive_unit(None, cnx=cnx)
             cnx.commit()
 
@@ -308,7 +388,7 @@ class SecurityTC(CubicWebTC):
             unit_alt_seq.reverse_seda_title[0].cw_set(title=u'gloup')
             cnx.commit()
 
-        with self.new_access('anon').client_cnx() as cnx:
+        with self.new_access('anon').cnx() as cnx:
             title = cnx.entity_from_eid(unit_alt_seq.reverse_seda_title[0].eid)
             unit = cnx.entity_from_eid(unit.eid)
             with self.assertUnauthorized(cnx):
@@ -321,11 +401,72 @@ class SecurityTC(CubicWebTC):
                 title.cw_delete()
             with self.assertUnauthorized(cnx):
                 unit.cw_delete()
-        with self.admin_access.repo_cnx() as cnx:
+        with self.admin_access.cnx() as cnx:
             unit = cnx.entity_from_eid(unit.eid)
             unit.cw_delete()
             cnx.commit()
 
+    def test_users_can_create_unit(self):
+        with self.new_access('alice').cnx() as cnx:
+            unit, unit_alt, unit_alt_seq = testutils.create_archive_unit(None, cnx=cnx)
+            cnx.commit()
+
+    def test_users_can_clone(self):
+        """Functional test for SEDA component clone."""
+        with self.admin_access.cnx() as cnx:
+            unit, unit_alt, unit_alt_seq = testutils.create_archive_unit(None, cnx=cnx)
+            cnx.commit()
+
+        with self.new_access('alice').cnx() as cnx:
+            unit = cnx.entity_from_eid(unit.eid)
+            cloned = cnx.create_entity(unit.cw_etype, user_annotation=u'x', clone_of=unit)
+            cnx.commit()
+            cloned.cw_clear_all_caches()
+
+            self.assertTrue(cloned.first_level_choice.content_sequence)
+
+    def test_scheme_type(self):
+        with self.new_access('alice').cnx() as cnx:
+            scheme = cnx.create_entity('ConceptScheme', title=u'Some vocabulary')
+            cnx.commit()
+
+            etype = cnx.find('CWEType', name=u'CWEType').one()
+            with self.assertUnauthorized(cnx):
+                scheme.cw_set(scheme_entity_type=etype)
+
+            rtype = cnx.find('CWRType', name=u'name').one()
+            with self.assertUnauthorized(cnx):
+                scheme.cw_set(scheme_relation_type=rtype)
+
+    def test_scheme_code_keyword_type(self):
+        with self.admin_access.cnx() as cnx:
+            admin_scheme = testutils.scheme_for_type(cnx, u'seda_keyword_type_to', None,
+                                                     u'type1')
+            cnx.commit()
+            type_concept = admin_scheme.reverse_in_scheme[0]
+
+        with self.new_access('alice').cnx() as cnx:
+            # ensure code_keyword_type is editable by standard user
+            scheme = cnx.create_entity('ConceptScheme', title=u'Some vocabulary',
+                                       code_keyword_type=type_concept)
+            cnx.commit()
+            scheme.cw_set(code_keyword_type=None)
+            cnx.commit()
+
+
+class SchemaIteratorTC(CubicWebTC):
+
+    def test_iter_all_rdefs(self):
+        values = {str(rdef) for rdef, role in iter_all_rdefs(self.schema, 'SEDABinaryDataObject')}
+        # composite relation
+        self.assertIn('relation SEDACreatingOs seda_creating_os SEDABinaryDataObject', values)
+        # external relation
+        self.assertIn('relation SEDARelationship seda_type_relationship Concept', values)
+        # outside this compound graph
+        self.assertNotIn(
+            'relation SEDADataObjectVersion seda_data_object_version_from SEDAPhysicalDataObject',
+            values)
+
 
 if __name__ == '__main__':
     import unittest
