@@ -19,6 +19,7 @@ from json import dumps
 import unittest
 
 from six import text_type
+from six.moves import zip
 
 from cubicweb.devtools.testlib import CubicWebTC
 from cubicweb.web import INTERNAL_FIELD_VALUE
@@ -26,7 +27,7 @@ from cubicweb.web import INTERNAL_FIELD_VALUE
 from cubicweb_seda.xsd2yams import RULE_TYPES
 from cubicweb_seda.views import archiveunit, clone, export, mgmt_rules
 
-import testutils
+from cubicweb_seda import testutils
 
 
 def entity_fields(fields):
@@ -36,7 +37,7 @@ def entity_fields(fields):
 
 class ManagementRulesTC(CubicWebTC):
     def test_rule_ref_vocabulary(self):
-        with self.admin_access.client_cnx() as cnx:
+        with self.admin_access.cnx() as cnx:
             create = cnx.create_entity
 
             access_scheme = create('ConceptScheme', title=u'access')
@@ -72,7 +73,7 @@ class ManagementRulesTC(CubicWebTC):
                               (access_concept.label(), text_type(access_concept.eid))])
 
     def test_archive_unit_component_rule_ref_vocabulary(self):
-        with self.admin_access.client_cnx() as cnx:
+        with self.admin_access.cnx() as cnx:
             unit, unit_alt, unit_alt_seq = testutils.create_archive_unit(None, cnx=cnx)
             for rule_type in ('access', 'appraisal'):
                 etype = 'SEDASeq{0}RuleRule'.format(rule_type.capitalize())
@@ -94,7 +95,7 @@ class PermissionsTC(CubicWebTC):
         """Check that a user cannot edit a SEDA profile he/she did not create."""
         alice_login = 'alice'
         bob_login = 'bob'
-        with self.admin_access.repo_cnx() as cnx:
+        with self.admin_access.cnx() as cnx:
             self.create_user(cnx, login=alice_login)
             self.create_user(cnx, login=bob_login)
             cnx.commit()
@@ -123,7 +124,7 @@ class RelationWidgetTC(CubicWebTC):
     """Functional test case about the relation widget."""
 
     def test_linkable_rset(self):
-        with self.admin_access.repo_cnx() as cnx:
+        with self.admin_access.cnx() as cnx:
             transfer = cnx.create_entity('SEDAArchiveTransfer', title=u'Test widget')
             bdo = cnx.create_entity('SEDABinaryDataObject',
                                     user_annotation=u'I am mandatory',
@@ -140,7 +141,7 @@ class RelationWidgetTC(CubicWebTC):
             view = self.vreg['views'].select('search_related_entities', req,
                                              rset=compressed.as_rset())
             self.failIf(view.linkable_rset())
-        with self.admin_access.repo_cnx() as cnx:
+        with self.admin_access.cnx() as cnx:
             scheme = cnx.create_entity('ConceptScheme', title=u'CompressionAlgorithm')
             scheme.add_concept(label=u'bz2')
             cnx.create_entity('SEDACompressionAlgorithmCodeListVersion',
@@ -279,7 +280,6 @@ class ArchiveTransferExportTC(CubicWebTC):
             req.cnx.commit()
             for version, fmt, expected_filename in (
                 ('2.0', 'rng', 'diagnosis testing-2.0.rng'),
-                ('2.0', 'xsd', 'diagnosis testing-2.0.xsd'),
                 ('2.0', 'html', 'diagnosis testing-2.0.html'),
                 ('1.0', 'rng', 'diagnosis testing-1.0.rng'),
                 ('1.0', 'xsd', 'diagnosis testing-1.0.xsd'),
@@ -297,7 +297,7 @@ class SimplifiedFormsTC(CubicWebTC):
     """Functional test case about forms in the web interface."""
 
     def setup_database(self):
-        with self.admin_access.repo_cnx() as cnx:
+        with self.admin_access.cnx() as cnx:
             transfer = cnx.create_entity('SEDAArchiveTransfer', title=u'diagnosis testing',
                                          simplified_profile=True)
             archive_unit = testutils.create_archive_unit(transfer)[0]
@@ -359,7 +359,7 @@ class SimplifiedFormsTC(CubicWebTC):
                          'InlineAddNewLinkView'),
                     ])
             form = self.create_and_link_form(
-                req, 'SEDAClassificationRule'.format(rule_type.capitalize()),
+                req, 'SEDAClassificationRule',
                 linkto=self.transfer_eid)
             self.assertInlinedFields(form, [
                 ('seda_seq_classification_rule_rule', 'InlineAddNewLinkView'),
@@ -386,8 +386,6 @@ class SimplifiedFormsTC(CubicWebTC):
                 self.assertInlinedFields(form, [
                     ('seda_seq_{0}_rule_rule'.format(rule_type),
                      'RuleRuleInlineEntityCreationFormView'),
-                    ('seda_alt_{0}_rule_prevent_inheritance'.format(rule_type),
-                     'InlineAddNewLinkView'),
                 ])
                 self.assertNoRemove(form, 'seda_seq_{0}_rule_rule'.format(rule_type), 'subject')
 
@@ -405,17 +403,6 @@ class SimplifiedFormsTC(CubicWebTC):
                 self.assertNoRemove(form, 'seda_start_date', 'object')
                 self.assertNoTitle(form, 'seda_start_date', 'object')
 
-    def test_prevent_inheritance_form(self):
-        with self.admin_access.web_request() as req:
-            for rule_type in ('access', 'appraisal'):
-                form = self.create_and_link_form(req, 'SEDAAlt{0}RulePreventInheritance'.format(
-                    rule_type.capitalize()))
-                self.assertInlinedFields(form, [
-                    ('seda_prevent_inheritance', 'PreventInheritanceInlineEntityCreationFormView'),
-                ])
-                self.assertNoRemove(form, 'seda_prevent_inheritance', 'object')
-                self.assertEqual(form.form_renderer_id, 'not-an-alt')
-
     def test_create_data_object_full(self):
         with self.admin_access.web_request() as req:
             req.entity_from_eid(self.transfer_eid).cw_set(simplified_profile=False)
@@ -529,9 +516,9 @@ class CloneImportTC(CubicWebTC):
                 {'p': eid})
             self.assertEqual(len(rset), 2)
             self.assertCountEqual([oeid for __, oeid in rset.rows], to_clone)
-            cardinalities, annotations = zip(*[
+            cardinalities, annotations = list(zip(*[
                 (clone.user_cardinality, clone.user_annotation)
-                for clone in rset.entities()])
+                for clone in rset.entities()]))
             self.assertCountEqual(cardinalities, ('1', '0..1'))
             self.assertCountEqual(annotations, ('plop', 'plouf'))
 
@@ -548,50 +535,85 @@ class SEDATreeTC(CubicWebTC):
     def test_archiveunit_reparent_to_transfer(self):
         with self.admin_access.cnx() as cnx:
             transfer = cnx.entity_from_eid(self.transfer_eid)
-            archunit, _, _ = testutils.create_archive_unit(transfer)
-            transfer2 = cnx.create_entity('SEDAArchiveTransfer',
-                                          title=u'Test2',
-                                          simplified_profile=True)
+            unit, unit_alt, unit_alt_seq = testutils.create_archive_unit(transfer)
+            unit2 = testutils.create_archive_unit(unit_alt_seq)[0]
             cnx.commit()
-            archunit.cw_clear_all_caches()
-            archunit.cw_adapt_to('IJQTree').reparent(transfer2.eid)
+            unit2.cw_adapt_to('IJQTree').reparent(transfer.eid, 0)
             cnx.commit()
-            transfer2.cw_clear_all_caches()
-            self.assertEqual([x.eid for x in transfer2.reverse_seda_archive_unit],
-                             [archunit.eid])
+            transfer.cw_clear_all_caches()
+            self.assertEqual([x.eid for x in transfer.reverse_seda_archive_unit],
+                             [unit2.eid, unit.eid])
 
     def test_archiveunit_reparent_to_archiveunit(self):
         with self.admin_access.cnx() as cnx:
             transfer = cnx.entity_from_eid(self.transfer_eid)
-            archunit, _, _ = testutils.create_archive_unit(transfer)
-            archunit2, _, _ = testutils.create_archive_unit(transfer)
+            unit, _, unit_alt_seq = testutils.create_archive_unit(transfer)
+            unit2, _, unit2_alt_seq = testutils.create_archive_unit(transfer)
+            subunit, _, _ = testutils.create_archive_unit(unit2_alt_seq)
             cnx.commit()
-            archunit.cw_clear_all_caches()
-            archunit.cw_adapt_to('IJQTree').reparent(archunit2.eid)
+            unit.cw_clear_all_caches()
+            unit.cw_adapt_to('IJQTree').reparent(unit2.eid, 0)
             cnx.commit()
-            seq = archunit2.first_level_choice.content_sequence
-            seq.cw_clear_all_caches()
-            self.assertEqual([x.eid for x in seq.reverse_seda_archive_unit],
-                             [archunit.eid])
+            unit2_alt_seq.cw_clear_all_caches()
+            self.assertEqual([x.eid for x in unit2_alt_seq.reverse_seda_archive_unit],
+                             [unit.eid, subunit.eid])
 
     def test_binarydataobject_reparent(self):
         with self.admin_access.cnx() as cnx:
             transfer = cnx.entity_from_eid(self.transfer_eid)
-            archunit, _, alt_seq = testutils.create_archive_unit(transfer)
-            archunit2, _, alt_seq2 = testutils.create_archive_unit(transfer)
+            unit, _, unit_alt_seq = testutils.create_archive_unit(transfer)
+            unit2, _, unit2_alt_seq = testutils.create_archive_unit(transfer)
             bdo = testutils.create_data_object(transfer)
             ref = cnx.create_entity('SEDADataObjectReference',
-                                    seda_data_object_reference=alt_seq,
+                                    seda_data_object_reference=unit_alt_seq,
                                     seda_data_object_reference_id=bdo)
             cnx.commit()
             bdo.cw_clear_all_caches()
-            bdo.cw_adapt_to('IJQTree').reparent(archunit2.eid)
+            bdo.cw_adapt_to('IJQTree').reparent(unit2.eid, 0)
             cnx.commit()
-            alt_seq2.cw_clear_all_caches()
-            self.assertEqual([x.eid for x in alt_seq2.reverse_seda_data_object_reference],
+            unit2_alt_seq.cw_clear_all_caches()
+            self.assertEqual([x.eid for x in unit2_alt_seq.reverse_seda_data_object_reference],
                              [ref.eid])
-            alt_seq.cw_clear_all_caches()
-            self.assertFalse(alt_seq.reverse_seda_data_object_reference)
+            unit_alt_seq.cw_clear_all_caches()
+            self.assertFalse(unit_alt_seq.reverse_seda_data_object_reference)
+
+    def test_binarydataobject_reorder_full(self):
+        with self.admin_access.cnx() as cnx:
+            transfer = cnx.entity_from_eid(self.transfer_eid)
+            transfer.cw_set(simplified_profile=False)
+            bdo1 = testutils.create_data_object(transfer)
+            bdo2 = testutils.create_data_object(transfer)
+            cnx.commit()
+
+            transfer.cw_adapt_to('IJQTree').move_child_at_index(bdo2.eid, 0)
+
+            transfer.cw_clear_all_caches()
+            self.assertEqual([x.eid for x in transfer.reverse_seda_binary_data_object],
+                             [bdo2.eid, bdo1.eid])
+
+            bdo2.cw_clear_all_caches()
+            transfer.cw_adapt_to('IJQTree').move_child_at_index(bdo2.eid, 2)
+
+            bdo1.cw_clear_all_caches()
+            bdo2.cw_clear_all_caches()
+
+            transfer.cw_clear_all_caches()
+            self.assertEqual([x.eid for x in transfer.cw_adapt_to('ITreeBase').iterchildren()],
+                             [bdo1.eid, bdo2.eid])
+
+    def test_binarydataobject_reorder_simplified(self):
+        with self.admin_access.cnx() as cnx:
+            transfer = cnx.entity_from_eid(self.transfer_eid)
+            unit, _, unit_alt_seq = testutils.create_archive_unit(transfer)
+            bdo1 = testutils.create_data_object(unit_alt_seq)
+            bdo2 = testutils.create_data_object(unit_alt_seq)
+            cnx.commit()
+
+            unit.cw_adapt_to('IJQTree').move_child_at_index(bdo2.eid, 0)
+
+            transfer.cw_clear_all_caches()
+            self.assertEqual([x.eid for x in unit.cw_adapt_to('ITreeBase').iterchildren()],
+                             [bdo2.eid, bdo1.eid])
 
 
 class FakeForm(object):
